"""Dataset classes compliant to the Pytorch standard."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import pathlib

# externals
import torch
import numpy as np
import xarray as xr

# locals
from climax.core.constants import ERA5_VARIABLES, PROJECTION
from pysegcnn.core.utils import search_files


class EoDataset(torch.utils.data.Dataset):

    @staticmethod
    def to_tensor(x, dtype=torch.float32):
        """Convert ``x`` to :py:class:`torch.Tensor`.

        Parameters
        ----------
        x : array_like
            The input data.
        dtype : :py:class:`torch.dtype`
            The data type used to convert ``x``.

            The modified class labels.
        Returns
        -------
        x : `torch.Tensor`
            The input data tensor.

        """
        return torch.tensor(np.asarray(x).copy(), dtype=dtype)


class NetCDFDataset(EoDataset):

    def __init__(self, X, y, dim='time'):

        # NetCDF dataset containing predictor variables (ERA5)
        self.X = X

        # NetCDF dataset containing target variable (OBS)
        self.y = y

        # dimension along which to sample
        self.dim = dim

    def __len__(self):
        return len(self.X[self.dim])

    def __getitem__(self, idx):
        return (self.to_tensor(self.X.isel({self.dim: idx}).to_array().values),
                self.to_tensor(self.y.isel({self.dim: idx}).to_array().values))


class ERA5Dataset(EoDataset):

    def __init__(self, root_dir, variables, plevels=None):

        # root directory: search for ERA5 files
        self.root_dir = pathlib.Path(root_dir)

        # valid variable names
        self.variables = [var for var in variables if var in ERA5_VARIABLES]

        # pressure levels
        self.plevels = plevels

    def merge(self, **kwargs):
        # search dataset for each variable in root directory
        datasets = [search_files(self.root_dir.joinpath(var), '.nc$').pop() for
                    var in self.variables]

        # iterate over input datasets and select pressure levels
        predictors = []
        for ds in datasets:
            # read dataset
            ds = xr.open_dataset(ds, **kwargs)

            # check if the dataset is defined on pressure levels or single
            # levels
            if 'level' in ds.dims:
                # check specified pressure levels to retrieve
                levels = (self.plevels if self.plevels is not None else
                          ds.level.values)

                # iterate over pressure levels to use
                for pl in levels:
                    # check if pressure level is available
                    if pl not in ds.level:
                        continue

                    # select pressure level and drop unnecessary dimensions
                    level = ds.sel(level=pl).drop('level')

                    # rename variable including corresponding pressure level
                    level = level.rename({k: '_'.join([k, str(pl)]) for k in
                                          level.data_vars if k != PROJECTION})

                    # append current pressure level to list of predictors
                    predictors.append(level)
            else:
                # append single level dataset to list of predictors
                predictors.append(ds)

        # drop projection variable, if present
        predictors = [ds.drop_vars(PROJECTION) for ds in predictors if
                      PROJECTION in ds.data_vars]

        # merge final predictor dataset
        return xr.merge(predictors)
