"""Dynamical climate downscaling using deep convolutional neural networks."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
from logging.config import dictConfig

# externals
import torch
import xarray as xr
from sklearn.model_selection import train_test_split
from torch.utils.data import DataLoader

# locals
from pysegcnn.core.utils import search_files
from pysegcnn.core.models import SegNet
from pysegcnn.core.trainer import NetworkTrainer
from pysegcnn.core.logging import log_conf
from climax.core.dataset import ERA5Dataset, NetCDFDataset
from climax.core.constants import ERA5_VARIABLES
from climax.main.config import (ERA5_PLEVELS, PREDICTAND, CALIB_PERIOD,
                                SHUFFLE, BATCH_SIZE, LR, TRAIN_CONFIG)
from climax.main.io import ERA5_PATH, OBS_PATH, MODEL_PATH


if __name__ == '__main__':

    # initialize logging
    dictConfig(log_conf())

    # initialize ERA5 predictor dataset
    Era5 = ERA5Dataset(ERA5_PATH.joinpath('ERA5'), ERA5_VARIABLES,
                       plevels=ERA5_PLEVELS)
    Era5_ds = Era5.merge()

    # initialize OBS predictand dataset
    Obs_ds = search_files(OBS_PATH.joinpath(PREDICTAND), '.nc$').pop()
    Obs_ds = xr.open_dataset(Obs_ds)

    # split calibration period into training and validation period
    train, valid = train_test_split(CALIB_PERIOD, shuffle=SHUFFLE)

    # training and validation dataset
    Era5_train, Obs_train = Era5_ds.sel(time=train), Obs_ds.sel(time=train)
    Era5_valid, Obs_valid = Era5_ds.sel(time=valid), Obs_ds.sel(time=valid)

    # create PyTorch compliant dataset and dataloader instances for model
    # training
    train_ds = NetCDFDataset(Era5_train, Obs_train, dim='time')
    valid_ds = NetCDFDataset(Era5_valid, Obs_valid, dim='time')
    train_dl = DataLoader(train_ds, batch_size=BATCH_SIZE, shuffle=SHUFFLE,
                          drop_last=False)
    valid_dl = DataLoader(valid_ds, batch_size=BATCH_SIZE, shuffle=SHUFFLE,
                          drop_last=False)

    # initialize network
    net = SegNet(MODEL_PATH.joinpath(PREDICTAND + '.pt'),
                 len(Era5_ds.data_vars), 1, filters=[32, 64, 128])

    # initialize optimizer
    optimizer = torch.optim.Adam(net.parameters(), lr=LR)

    # initialize loss function
    loss_function = torch.nn.MSELoss()

    # initialize network trainer
    trainer = NetworkTrainer(net, optimizer, net.state_file, train_dl,
                             valid_dl, loss_function=loss_function,
                             **TRAIN_CONFIG)

    # train model
    state = trainer.train()
